#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from warnings import warn
warn(
    "woost.controllers.language.LanguageModule is deprecated, "
    "use woost.languagescheme.LanguageScheme instead "
    "(or its global instance through woost.app.language)",
    DeprecationWarning
)

from woost.languagescheme import LanguageScheme as LanguageModule




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os.path
from cocktail.controllers import serve_file
from woost.models import File
from woost.controllers.basecmscontroller import BaseCMSController
from woost.extensions.audio.request import request_encoding


class AudioEncodingController(BaseCMSController):

    def __call__(self, file_id, encoder_id):
        file = File.require_instance(int(file_id))
        encoder_id = os.path.splitext(encoder_id)[0]
        dest = request_encoding(file, encoder_id)
        return serve_file(dest)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers import (
    request_property,
    get_parameter,
    Controller,
    FormProcessor,
    Form
)
from woost.extensions.ecommerce.ecommerceproduct import ECommerceProduct
from woost.extensions.ecommerce.productcontroller import ProductController


class ECommerceProductListingController(FormProcessor, Controller):

    class AddProductForm(ProductController.AddProductForm):

        @request_property
        def product(self):
            return get_parameter(
                schema.Reference(
                    "product",
                    type = ECommerceProduct,
                    required = True
                )
            )

        @request_property
        def model(self):
            return self.product \
                and self.product.purchase_model \
                or Form.model(self)

        @request_property
        def adapter(self):
            return self.product \
                and ProductController.AddProductForm.adapter(self) \
                or Form.adapter(self)

        def create_instance(self):
            return self.product \
                and ProductController.AddProductForm.create_instance(self) \
                or Form.create_instance(self)




#-*- coding: utf-8 -*-
u"""Defines the `BlocksPage` model.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.page import (
    Page as BlocksPage,
    PageTranslation as BlocksPageTranslation
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.twittertimelineblock import (
    TwitterTimelineBlock,
    TwitterTimelineBlockTranslation
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.schema.exceptions import ValidationError
from woost import app
from woost.models import Publishable

default_document = object()

def requires_agreement(form, name = "terms", document = default_document):

    if document is default_document:
        document = "%s.%s" % (app.package, name)

    if isinstance(document, basestring):
        document = Publishable.require_instance(qname = document)

    member = FormAgreement(
        name = name,
        agreement_document = document
    )

    form.schema.add_member(member)

    if form.schema.groups_order:
        if "form_agreement" not in form.schema.groups_order:
            if not isinstance(form.schema.groups_order, list):
                form.schema.groups_order = list(form.schema.groups_order)
            form.schema.groups_order.append("form_agreement")

    form.adapter.exclude(name)
    return member


class FormAgreement(schema.Boolean):

    agreement_document = None

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("required", True)
        kwargs.setdefault("member_group", "form_agreement")
        schema.Boolean.__init__(self, *args, **kwargs)
        self.add_validation(FormAgreement.consent_validation)

    def consent_validation(self, value, context):
        if not value:
            yield ConsentNotGivenError(self, value, context)

    def __translate__(self, language, **kwargs):
        return (
            schema.Boolean.__translate__(self, language, **kwargs)
            or translations(
                "woost.controllers.formagreement.%s" % self.name,
                member = self
            )
            or translations("woost.controllers.formagreement", member = self)
        )

    def parse_request_value(self, reader, value):
        return schema.Boolean.parse_request_value(self, reader, value) or False


class ConsentNotGivenError(ValidationError):
    """A validation error produced by forms that enforce an agreement to a
    terms and conditions document when the user attempts to submit the form
    without acknowledging the agreement.
    """




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .block import Block
from .publishable import Publishable


class FacebookLikeButton(Block):

    instantiable = True
    type_group = "blocks.social"
    view_class = "cocktail.html.FacebookLikeButton"

    members_order = [
        "fb_href",
        "fb_layout",
        "fb_send",
        "fb_show_faces",
        "fb_width",
        "fb_action",
        "fb_font",
        "fb_colorscheme",
        "fb_ref"
    ]

    fb_href = schema.Reference(
        type = Publishable,
        related_end = schema.Collection(),
        member_group = "content"
    )

    fb_layout = schema.String(
        required = True,
        default = "standard",
        enumeration = ["standard", "button_count", "box_count"],
        member_group = "appearence"
    )

    fb_send = schema.Boolean(
        default = False,
        required = True,
        member_group = "appearence"
    )

    fb_show_faces = schema.Boolean(
        default = False,
        required = True,
        member_group = "appearence"
    )

    fb_width = schema.Integer(
        required = True,
        default = 450,
        member_group = "appearence"
    )

    fb_action = schema.String(
        required = True,
        default = "like",
        enumeration = ["like", "recommend"],
        member_group = "appearence"
    )

    fb_font = schema.String(
        required = True,
        default = "arial",
        enumeration = [
            "arial",
            "lucida grande",
            "segoe ui",
            "tahoma",
            "trebuchet ms",
            "verdana"
        ],
        translatable_enumeration = False,
        member_group = "appearence"
    )

    fb_colorscheme = schema.String(
        required = True,
        default = "light",
        enumeration = ["light", "dark"],
        member_group = "appearence"
    )

    fb_ref = schema.String(
        member_group = "content"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.href = self.fb_href and self.fb_href.get_uri(host = ".")
        view.send = self.fb_send
        view.layout = self.fb_layout
        view.show_faces = self.fb_show_faces
        view.width = self.fb_width
        view.action = self.fb_action
        view.font = self.fb_font
        view.colorscheme = self.fb_colorscheme
        view.ref = self.fb_ref




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.html import templates
from cocktail.translations import translations
from woost.models import Style

TinyMCE = templates.get_class("cocktail.html.TinyMCE")


class RichTextEditor(TinyMCE):

    styles = None
    tinymce_params = {
        "setup": "woost.richTextEditor.init",
        "plugins": "table link image fullscreen code "
                   "visualblocks contextmenu paste",
        "entity_encoding": "raw",
        "style_formats_merge": True,
        "height": 250,
        "content_css": "/user_styles/?backoffice=1",
        "statusbar": False,
        "menubar": [
            "edit",
            "insert",
            "table",
            "view"
        ],
        "menu": {
            "edit": {
                "title": "Edit",
                "items": "undo redo | "
                "cut copy paste pastetext | "
                "selectall | "
                "removeformat"
            },
            "insert": {
                "title": "Insert",
                "items": "image link"
            },
            "table": {
                "title": "Table",
                "items": "inserttable tableprops deletetable | cell row column"
            },
            "view": {
                "title": "View",
                "items": "visualaid visualblocks code fullscreen"
            }
        },
        "toolbar": "undo redo | "
                   "styleselect | "
                   "bold italic | "
                   "bullist numlist outdent indent"
    }

    def _build(self):
        TinyMCE._build(self)
        self.add_resource("/resources/scripts/RichTextEditor.js")

    def _get_default_styles(self):
        return list(Style.select({"applicable_to_text": True}))

    def _ready(self):

        styles = self.styles
        if styles is None:
            styles = self._get_default_styles()

        if styles:
            self.tinymce_params.setdefault("style_formats", [{
                "title": translations(
                    "woost.views.RichTextEditor.styles_menu_entry"
                ),
                "items": [
                    {
                        "title": translations(style),
                        "inline": "span",
                        "classes": style.class_name
                    }
                    for style in styles
                ]
            }])

        TinyMCE._ready(self)




#-*- coding: utf-8 -*-
u"""
Provides base and default content types for the woost CMS.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from cocktail import schema
from cocktail.events import when
from . import memberextensions

# Register the 'text/javascript' MIME type
import mimetypes
if not mimetypes.guess_extension("text/javascript"):
    mimetypes.add_type("text/javascript", ".js")
del mimetypes

@when(schema.RelationMember.attached_as_orphan)
def _hide_self_contained_relations(event):
    if event.anonymous:
        event.source.visible = False
        event.source.synchronizable = False

from woost.models.typegroups import (
    TypeGroup,
    type_groups,
    block_type_groups
)

# Base content types
#------------------------------------------------------------------------------
from .slot import Slot
from .localemember import LocaleMember
from .configuration import Configuration
from .website import Website
from .websitesession import (
    get_current_website,
    set_current_website
)
from .siteinstallation import SiteInstallation
from .changesets import ChangeSet, Change, changeset_context
from .item import Item
from .userview import UserView
from .style import Style
from .publishable import (
    Publishable,
    IsPublishedExpression,
    IsAccessibleExpression
)
from .document import Document
from .template import Template
from .controller import Controller
from .user import (
    User,
    AuthorizationError
)
from .usersession import (
    get_current_user,
    set_current_user
)
from .role import Role
from .permission import (
    Permission,
    ContentPermission,
    CreatePermission,
    ReadPermission,
    ModifyPermission,
    DeletePermission,
    RenderPermission,
    MemberPermission,
    ReadMemberPermission,
    ModifyMemberPermission,
    CreateTranslationPermission,
    ReadTranslationPermission,
    ModifyTranslationPermission,
    DeleteTranslationPermission,
    ReadHistoryPermission,
    InstallationSyncPermission,
    restricted_modification_context,
    delete_validating,
    PermissionExpression,
    ChangeSetPermissionExpression
)
from .page import Page
from .standardpage import StandardPage
from .file import File
from .news import News
from .event import Event
from .uri import URI
from .file import File
from .extension import (
    Extension,
    extension_translations,
    load_extensions
)
from .trigger import (
    Trigger,
    ContentTrigger,
    CreateTrigger,
    InsertTrigger,
    ModifyTrigger,
    DeleteTrigger
)
from .triggerresponse import (
    TriggerResponse,
    CustomTriggerResponse,
    SendEmailTriggerResponse
)
from .emailtemplate import EmailTemplate
from .feed import Feed

from .userfilter import (
    IsPublishedFilter,
    TypeFilter
)

from .caching import CachingPolicy

from . import rendering
from .videoplayersettings import VideoPlayerSettings
from .synchronization import (
    Synchronization,
    get_manifest,
    rebuild_manifest
)
from . import staticpublication

# Blocks
from .elementtype import ElementType
from .slot import Slot
from .block import Block
from .containerblock import ContainerBlock
from .blockimagefactoryreference import BlockImageFactoryReference
from .customblock import CustomBlock
from .eventlisting import EventListing
from .facebooklikebox import FacebookLikeBox
from .facebooklikebutton import FacebookLikeButton
from .flashblock import FlashBlock
from .htmlblock import HTMLBlock
from .iframeblock import IFrameBlock
from .loginblock import LoginBlock
from .menublock import MenuBlock
from .newslisting import NewsListing
from .publishablelisting import PublishableListing
from .slideshowblock import SlideShowBlock
from .textblock import TextBlock
from .tweetbutton import TweetButton
from .twittertimelineblock import TwitterTimelineBlock
from .videoblock import VideoBlock

from . import migration




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from woost.models import Publishable, File, Configuration

_get_uri = Publishable.get_uri

def get_uri(
    self,
    path = None,
    parameters = None,
    language = None,
    host = None,
    encode = True
):
    if isinstance(self, File) and host in (None, "!", "?"):
        host = Configuration.instance.get_setting("external_files_host")

    return _get_uri(
        self,
        path = path,
        parameters = parameters,
        language = language,
        host = host,
        encode = encode
    )

Publishable.get_uri = get_uri




#-*- coding: utf-8 -*-
u"""Defines the `URLShortener` model.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import abstractmethod
from cocktail import schema
from cocktail.caching import CacheKeyError
from woost.models import Item
from woost.extensions.shorturls import ShortURLsExtension


class URLShortener(Item):
    """Base model for all URL shortening services."""

    visible_from_root = False
    instantiable = False

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        descriptive = True
    )

    def request(self, url):

        short_url = None
        cache_key = (self.id, url)

        if short_url is None:
            try:
                short_url = ShortURLsExtension.url_cache.retrieve(cache_key)
            except CacheKeyError:
                short_url = self.shorten_url(url)
                ShortURLsExtension.url_cache.store(cache_key, short_url)

        return short_url

    @abstractmethod
    def shorten_url(self, url):
        pass


class URLShortenerServiceError(Exception):
    """An exception raised if an error is reported during a request to an URL
    shortening service.
    """

    def __init__(self, response):
        Exception.__init__(self, response)
        self.response = response




#-*- coding: utf-8 -*-
"""

@author:		Marc Pérez
@contact:		marc.perez@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
from cocktail.translations import translations

translations.define("googlesearch next",
    es = u"Siguiente",
    en = u"Next",
    fr = u"Suivante",
    ca = u"Següent"
)

translations.define("googlesearch previous",
    es = u"Anterior",
    en = u"Previous",
    fr = u"Dernière",
    ca = u"Anterior"
)

# Configuration
#------------------------------------------------------------------------------
translations.define("Configuration.services.google_cse",
    ca = u"Google CSE",
    es = u"Google CSE",
    en = u"Google CSE"
)

translations.define("Configuration.google_search_engine_id",
    ca = u"Identificador del cercador",
    es = u"Indentificador del buscador",
    en = u"Search engine id"
)

translations.define("Configuration.google_search_results_page",
    ca = u"Pàgina de resultats de cerca",
    es = u"Página de resultados de búsqueda",
    en = u"Search results page"
)

# Website
#------------------------------------------------------------------------------
translations.define("Website.services.google_cse",
    ca = u"Google CSE",
    es = u"Google CSE",
    en = u"Google CSE"
)

translations.define("Website.google_search_engine_id",
    ca = u"Identificador del cercador",
    es = u"Indentificador del buscador",
    en = u"Search results page"
)

translations.define("Website.google_search_results_page",
    ca = u"Pàgina de resultats de cerca",
    es = u"Página de resultados de búsqueda",
    en = u"Search results page"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .block import Block
from .publishable import Publishable


class FacebookLikeBox(Block):

    instantiable = True
    type_group = "blocks.social"
    view_class = "cocktail.html.FacebookLikeBox"

    members_order = [
        "fb_href",
        "fb_show_faces",
        "fb_stream",
        "fb_header",
        "fb_width",
        "fb_height",
        "fb_border_color",
        "fb_colorscheme"
    ]

    fb_href = schema.URL(
        required = True,
        member_group = "content"
    )

    fb_show_faces = schema.Boolean(
        default = True,
        required = True,
        member_group = "appearence"
    )

    fb_stream = schema.Boolean(
        default = False,
        required = True,
        member_group = "appearence"
    )

    fb_header = schema.Boolean(
        default = False,
        required = True,
        member_group = "appearence"
    )

    fb_width = schema.Integer(
        required = True,
        default = 292,
        member_group = "appearence"
    )

    fb_height = schema.Integer(
        member_group = "appearence"
    )

    fb_border_color = schema.Color(
        member_group = "appearence"
    )

    fb_colorscheme = schema.String(
        required = True,
        default = "light",
        enumeration = ["light", "dark"],
        member_group = "appearence"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.href = self.fb_href
        view.show_faces = self.fb_show_faces
        view.stream = self.fb_stream
        view.header = self.fb_header
        view.width = self.fb_width
        view.height = self.fb_height
        view.border_color = self.fb_border_color
        view.colorscheme = self.fb_colorscheme




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from glob import glob
from shutil import rmtree, copy
from cocktail.events import when
from cocktail.persistence import datastore
from woost import app
from woost.models.item import Item
from woost.models.user import User
from woost.models.permission import RenderPermission
from woost.models.rendering.imagefactory import ImageFactory
from woost.models.rendering.formats import (
    mime_types_by_format,
    extensions_by_format,
    formats_by_extension,
    default_format
)

debug = False

def _remove_dir_contents(path, pattern = None):

    # os.path.lexists is not supported on windows
    from cocktail.styled import styled
    exists = getattr(os.path, "lexists", os.path.exists)

    if exists(path):

        if pattern:
            items = glob(os.path.join(path, pattern))
        else:
            items = os.listdir(path)

        for item in items:
            item_path = os.path.join(path, item)
            if debug:
                print styled(" " * 4 + item_path, "red")
            try:
                if os.path.isdir(item_path):
                    rmtree(item_path)
                else:
                    os.remove(item_path)
            except OSError, ex:
                if debug:
                    print styled(ex, "red")

def clear_image_cache(item = None, factory = None):

    if not app.root:
        return

    if debug:
        from cocktail.styled import styled
        print styled("Clearing image cache", "red"),

        if item:
            print styled("Item:", "light_gray"),
            print styled(item, "red", style = "bold"),

        if factory:
            print styled("Factory:", "light_gray"),
            print styled(factory, "red", style = "bold"),

        print

    # Remove the full cache
    if item is None and factory is None:
        _remove_dir_contents(app.path("image-cache"))
        _remove_dir_contents(app.path("static", "images"))

    # Selective drop: per item and/or factory
    else:
        paths = []

        if item is not None:
            paths.append(app.path("image-cache", str(item.id)))
            paths.append(app.path("static", "images", str(item.id)))
        else:
            for base in (
                app.path("image-cache"),
                app.path("static", "images")
            ):
                for item in os.listdir(base):
                    path = os.path.join(base, item)
                    if os.path.isdir(path):
                        paths.append(path)

        if factory is None:
            pattern = None
        else:
            pattern = factory + ".*"

        for path in paths:
            _remove_dir_contents(path, pattern)

@when(Item.changed)
@when(Item.deleted)
def _clear_image_cache_after_commit(event):
    if app.root:
        item = event.source
        if item.is_inserted:
            clear_image_cache_after_commit(item = item)

def clear_image_cache_after_commit(item = None, factory = None):

    if not app.root:
        return

    if factory:
        factory_identifier = factory.identifier or "factory%d" % factory.id
    else:
        factory_identifier = None

    key = "woost.models.rendering.clear_image_cache(item=%s,factory=%s)" % (
        item.id if item else "*",
        factory_identifier or "*"
    )

    datastore.unique_after_commit_hook(
        key,
        _clear_image_cache_after_commit_callback,
        item,
        factory_identifier
    )

def _clear_image_cache_after_commit_callback(commit_successful, item, factory):
    if commit_successful:
        clear_image_cache(item, factory)

def require_rendering(
    item,
    factory = None,
    format = None,
    parameters = None):

    ext = None

    if factory is None:
        factory = ImageFactory.require_instance(identifier = "default")

    if format is None:
        if not isinstance(item, type):
            ext = getattr(item, "file_extension", None)

            if ext is not None:
                format = formats_by_extension.get(ext.lstrip(".").lower())
                if format is None:
                    ext = None

    elif format not in mime_types_by_format:
        raise BadRenderingRequest("Invalid image format: %s" % format)

    if format is None:
        format = default_format

    if ext is None:
        ext = extensions_by_format[format]

    identifier = factory.identifier or "factory%d" % factory.id
    file_name = "%s.%s" % (identifier, ext)
    item_id = item.full_name if isinstance(item, type) else str(item.id)

    # If the image hasn't been generated yet, do so and store it in the
    # application's image cache
    image_cache_file = app.path("image-cache", item_id, file_name)

    if not os.path.exists(image_cache_file):

        # Generate the file
        image = factory.render(item)

        if not image:
            raise RenderError("Can't render %s" % item)

        # Store the generated image in the image cache
        try:
            os.mkdir(app.path("image-cache", item_id))
        except OSError:
            pass

        if isinstance(image, basestring):
            try:
                os.remove(image_cache_file)
            except OSError:
                pass

            if hasattr(os, "symlink"):
                os.symlink(image, image_cache_file)
            else:
                copy(image, image_cache_file)
        else:
            if format == 'JPEG' and image.mode not in ('RGB', 'RGBA'):
                image = image.convert('RGBA')
            elif format == 'PNG' and image.mode == "CMYK":
                image = image.convert('RGBA')
            image.save(image_cache_file, format)

    # If the image is accessible to anonymous users, create a link in the
    # application's static content folder (further requests will be served
    # by the web server, no questions asked).
    if hasattr(os, "symlink"):
        static_publication_link = \
            app.path("static", "images", item_id, file_name)

        if not os.path.lexists(static_publication_link):
            anonymous = User.require_instance(qname = "woost.anonymous_user")

            if anonymous.has_permission(
                RenderPermission,
                target = item,
                image_factory = factory
            ):
                try:
                    os.mkdir(app.path("static", "images", item_id))
                except OSError:
                    pass
                os.symlink(image_cache_file, static_publication_link)

    return image_cache_file


class BadRenderingRequest(Exception):
    """An exception raised when trying to render a piece of content using
    invalid parameters (ie. an unknown image format).
    """


class RenderError(Exception):
    """An exception raised when a request for a certain image can't be
    fulfilled (ie. because there is no content renderer that can handle the
    specified content).
    """




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2013
"""
from cocktail.events import event_handler
from cocktail import schema
from .item import Item
from .user import User


class SiteInstallation(Item):

    type_group = "setup"
    edit_form = "woost.views.SiteInstallationForm"

    edit_node_class = (
        "woost.controllers.backoffice."
        "siteinstallationeditnode.SiteInstallationEditNode"
    )

    members_order = (
        "title",
        "url",
        "synchronization_user",
        "synchronization_password"
    )

    default_synchronizable = False

    title = schema.String(
        indexed = True,
        unique = True,
        descriptive = True,
        required = True
    )

    url = schema.URL(
        required = True
    )

    synchronization_user = schema.String(
        listed_by_default = False
    )

    synchronization_password = schema.String(
        listed = False,
        listed_by_default = False,
        visible_in_detail_view = False,
        searchable = False,
        text_search = False,
        edit_control = "cocktail.html.PasswordBox"
    )

    @event_handler
    def handle_changing(cls, event):

        if event.member is cls.synchronization_password \
        and event.value is not None:
            event.value = User.encryption(event.value)

    def test_password(self, password):
        """Indicates if the user's password matches the given string.

        @param password: An unencrypted string to tests against the user's
            encrypted password.
        @type password: str

        @return: True if the passwords match, False otherwise.
        @rtype: bool
        """
        if password:
            return self.encryption(password) == self.synchronization_password
        else:
            return not self.synchronization_password




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations


# Configuration
#------------------------------------------------------------------------------
translations.define("Configuration.services.campaign_monitor",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

translations.define("Configuration.campaign_monitor_api_key",
    ca = u"Clau d'API",
    es = u"Clave de API",
    en = u"API key"
)

translations.define("Configuration.campaign_monitor_client_id",
    ca = u"ID de client",
    es = u"ID de cliente",
    en = u"Client ID"
)

# Website
#------------------------------------------------------------------------------
translations.define("Website.services.campaign_monitor",
    ca = u"Campaign Monitor",
    es = u"Campaign Monitor",
    en = u"Campaign Monitor"
)

translations.define("Website.campaign_monitor_api_key",
    ca = u"Clau d'API",
    es = u"Clave de API",
    en = u"API key"
)

translations.define("Website.campaign_monitor_client_id",
    ca = u"ID de client",
    es = u"ID de cliente",
    en = u"Client ID"
)

# CampaignMonitorList
#------------------------------------------------------------------------------
translations.define("CampaignMonitorList",
    ca = u"Llista de subscripció",
    es = u"Lista de suscripción",
    en = u"Subscription list"
)

translations.define("CampaignMonitorList-plural",
    ca = u"Llistes de subscripció",
    es = u"Listas de suscripción",
    en = u"Subscription lists"
)

translations.define("CampaignMonitorList.list_id",
    ca = u"Id de llista",
    es = u"Id de lista",
    en = u"List id"
)

translations.define("CampaignMonitorList.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("CampaignMonitorList.confirmation_page",
    ca = u"Pàgina de confirmació inicial",
    es = u"Página de confirmación inicial",
    en = u"Initial confirmation page"
)

translations.define("CampaignMonitorList.confirmation_page-explanation",
    ca = u"""
Pàgina que es mostrarà després de completar el formulari de subscripció. Si no
s'especifica una pàgina, es mostrarà una pàgina genèrica amb instruccions.
""",
    es = u"""
Página que se mostrará después de completar el formulario de suscripción. Si no
se especifica una página, se mostrará una página genérica con instrucciones.
""",
    en = u"""
The page that will be displayed after someone completes the subscribe form. If
you don't specify a page, a generic page with instructions will be displayed to
the subscriber.
"""
)

translations.define("CampaignMonitorList.confirmation_success_page",
    ca = u"Pàgina de confirmació d'alta",
    es = u"Página de confirmación de alta",
    en = u"Confirmation success page"
)

translations.define("CampaignMonitorList.confirmation_success_page-explanation",
    ca = u"""
Pàgina que es mostrarà després que un subscriptor faci clic a l'enllaç de
confirmació en el correu de verificació. Si no s'especifica una pàgina, es
mostrarà una pàgina de gràcies genèrica.
""",
    es = u"""
Página que se mostrará después de que un suscriptor haga clic en el enlace de
confirmación en el correo de verificación. Si no se especifica una página, se
mostrará una página de gracias genérica.
""",
    en = u"""
Page that will be displayed after the subscriber clicks the confirmation link
in the verification email. If you don't specify a page, a generic thank you
page will be displayed to the subscriber.
"""
)

translations.define("CampaignMonitorList.unsubscribe_page",
    ca = u"Pàgina de baixa",
    es = u"Página de baja",
    en = u"Unsubscribe page"
)

translations.define("CampaignMonitorList.unsubscribe_page-explanation",
    ca = u"""
Pàgina que es mostrarà després que un subscriptor faci clic a l'enllaç de baixa
en un correu. Si no s'especifica una pàgina, es mostrarà una pàgina de
confirmació generica.
""",
    es = u"""
Página que se mostrará después de que un suscriptor haga clic en el enlace de
baja en un correo. Si no se especifica una página, se mostrará una pàgina de
confirmación generica.
""",
    en = u"""
Page that will be displayed after someone clicks an unsubscribe link in your
email. If you don't specify a page, a generic confirmation page will be
displayed to the subscriber.
"""
)

# CampaignMonitorListEditNode
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.campaignmonitorlisteditnode.CampaignMonitorListEditNode synchronized changes",
    ca = lambda item:
        u"Canvis a <strong>%s</strong> sincronitzats" % (
            translations(item, "ca"),
        ),
    es = lambda item:
        u"Cambios en <strong>%s</strong> sincronizados" % (
            translations(item, "es"),
        ),
    en = lambda item:
        u"Synchronized changes to <strong>%s</strong>" % (
            translations(item, "en"),
        )
)

translations.define(
    "woost.extensions.campaignmonitorlisteditnode.CampaignMonitorListEditNode failed synchronization",
    ca = lambda item, exception:
        u"Sincronització de <strong>%s</strong> fallida. <em>%s</em>" % (
            translations(item, "ca"),
            unicode(exception)
        ),
    es = lambda item, exception:
        u"Sincronización de <strong>%s</strong> fallida. <em>%s</em>" % (
            translations(item, "es"),
            unicode(exception)
        ),
    en = lambda item, exception:
        u"<strong>%s</strong> synchronization failed. <em>%s</em>" % (
            translations(item, "en"),
            unicode(exception)
        )
)

# Subscriber
#------------------------------------------------------------------------------
translations.define("Subscriber.subscriber_name",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("Subscriber.subscriber_email",
    ca = u"Correu electrònic",
    es = u"Correo electrónico",
    en = u"Email"
)

translations.define("subscription_form.lists",
    ca = u"Llistes",
    es = u"Listas",
    en = u"Lists"
)

# SubscriptionFormBlock
#------------------------------------------------------------------------------
translations.define("SubscriptionFormBlock",
    ca = u"Formulari de subscripció",
    es = u"Formulario de suscripción",
    en = u"Subscription form"
)

translations.define("SubscriptionFormBlock-plural",
    ca = u"Formulari de subscripció",
    es = u"Formulario de suscripción",
    en = u"Subscription form"
)

translations.define("SubscriptionFormBlock.subscriber_model",
    ca = u"Model de subscriptor",
    es = u"Modelo de suscriptor",
    en = u"Subscription model"
)

translations.define("SubscriptionFormBlock.lists",
    ca = u"Llistes",
    es = u"Listas",
    en = u"Lists"
)

translations.define("SubscriptionFormBlock.view_class",
    ca = u"Vista",
    es = u"Vista",
    en = u"View"
)

# ResubscribeFormView
#------------------------------------------------------------------------------
translations.define("woost.extensions.campaign3.ResubscribeFormView.unsubscribed_by_error",
    ca = u"<strong>No volies donar-te de baixa?</strong> Clica a continuació per seguir rebent els nostres missatges.",
    es = u"<strong>No querías darte de baja?</strong> Haz clic a continuación para seguir recibiendo nuestros mensajes.",
    en = u"<strong>You did not want to unsubscribe?</strong> Click below to continue receiving our messages."
)

translations.define("woost.extensions.campaign3.ResubscribeFormView.resubscribe",
    ca = u"Tornar a subscriure's",
    es = u"Volver a suscribirse",
    en = u"Re-subscribe"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations
from woost.models import Extension, Configuration
from woost.models.rendering import ChainRenderer


translations.define("TV3ALaCartaExtension",
    ca = u"TV3 a la carta",
    es = u"TV3 a la carta",
    en = u"TV3 a la carta"
)

translations.define("TV3ALaCartaExtension-plural",
    ca = u"TV3 a la carta",
    es = u"TV3 a la carta",
    en = u"TV3 a la carta"
)


class TV3ALaCartaExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afegeix suport per vídeos del 'TV3 a la carta'""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para videos del servicio 'TV3 a la carta'""",
            "es"
        )
        self.set("description",
            u"""Adds support for videos from the 'TV3 a la carta' service""",
            "en"
        )

    def _load(self):
        from woost.extensions.tv3alacarta import (
            strings,
            tv3alacartavideo,
            videoplayersettings
        )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from simplejson import dumps
from cocktail.translations import translations, get_language, set_language
from cocktail import schema
from cocktail.controllers import get_parameter, request_property
from cocktail.html import Element, templates
from woost.models import get_current_user, ReadPermission, Publishable
from woost.controllers.backoffice.basebackofficecontroller \
    import BaseBackOfficeController


class RenderPreviewController(BaseBackOfficeController):

    @request_property
    def previewed_item(self):
        return self.stack_node.item

    @request_property
    def preview_publishable(self):
        return get_parameter(
            schema.Reference("preview_publishable",
                type = Publishable
            )
        )

    @request_property
    def preview_language(self):
        return get_parameter(
            schema.String("preview_language",
                default = get_language()
            )
        )

    def __call__(self, *args, **kwargs):

        node = self.stack_node
        previewed_item = self.previewed_item
        publishable = self.preview_publishable
        preview_language = self.preview_language
        user = get_current_user()

        # Set the language for the preview
        if preview_language:
            set_language(preview_language)

        # Enforce permissions
        user.require_permission(ReadPermission, target = previewed_item)

        if publishable is not previewed_item:
            user.require_permission(ReadPermission, target = publishable)

        # Disable the preview if the item's unsaved state produces validation
        # errors; these would usually lead to unhandled server errors during
        # rendering.
        errors = schema.ErrorList(node.iter_errors())

        if errors:
            error_box = templates.new("cocktail.html.ErrorBox")
            error_box.errors = errors
            message = Element("div",
                class_name = "preview-error-box",
                children = [
                    translations(
                        "woost.backoffice invalid item preview",
                        preview_language
                    ),
                    error_box
                ]
            )
            message.add_resource("/resources/styles/backoffice.css")
            return message.render_page()

        # Update the edited item with the data to preview
        node.import_form_data(node.form_data, previewed_item)

        self.context.update(
            original_publishable = self.context["publishable"],
            publishable = publishable
        )

        controller = publishable.resolve_controller()

        if controller is None:
            raise cherrypy.NotFound()

        if isinstance(controller, type):
            controller = controller()

        return controller()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from warnings import warn
from cocktail.translations import translations
from woost.models import Extension


translations.define("BlocksExtension",
    ca = u"Blocs de contingut",
    es = u"Bloques de contenido",
    en = u"Content blocks"
)

translations.define("BlocksExtension-plural",
    ca = u"Blocs de contingut",
    es = u"Bloques de contenido",
    en = u"Content blocks"
)


class BlocksExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet la creació de contingut utilitzant blocs""",
            "ca"
        )
        self.set("description",
            u"""Permite la creación de contenido usando bloques""",
            "es"
        )
        self.set("description",
            u"""Allows the creation of content using blocs""",
            "en"
        )

    def _load(self):
        warn(
            "The blocks extension has been integrated with the core and is "
            "no longer necessary",
            DeprecationWarning
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .block import Block


class TwitterTimelineBlock(Block):

    instantiable = True
    type_group = "blocks.social"
    view_class = "cocktail.html.TwitterTimeline"

    members_order = [
        "widget_id",
        "theme",
        "link_color",
        "width",
        "height",
        "related_accounts"
    ]

    widget_id = schema.String(
        required = True,
        member_group = "content"
    )

    theme = schema.String(
        required = True,
        default = "light",
        enumeration = ("light", "dark"),
        member_group = "appearence"
    )

    link_color = schema.Color(
        member_group = "appearence"
    )

    width = schema.Integer(
        member_group = "appearence"
    )

    height = schema.Integer(
        member_group = "appearence"
    )

    related_accounts = schema.String(
        member_group = "tweet"
    )

    def init_view(self, view):
        Block.init_view(self, view)
        view.widget_id = self.widget_id
        view.theme = self.theme
        view.link_color = self.link_color
        view.width = self.width
        view.height = self.height
        view.related = self.related_accounts




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.blockutils import (
    find_blocks,
    create_block_views,
    type_is_block_container,
    add_block
)




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import buffet
from mimetypes import guess_type
from email.mime.text import MIMEText
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEBase import MIMEBase
from email.Header import Header
from email.Utils import formatdate, parseaddr, formataddr
from email import Encoders
from cocktail import schema
from cocktail.translations import language_context
from .item import Item
from .configuration import Configuration
from .file import File


class EmailTemplate(Item):

    type_group = "customization"
    encoding = "utf-8"

    members_order = [
        "title",
        "mime_type",
        "sender",
        "receivers",
        "bcc",
        "template_engine",
        "subject",
        "body",
        "language",
        "initialization_code"
    ]

    title = schema.String(
        listed_by_default = False,
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    mime_type = schema.String(
        required = True,
        default = "html",
        listed_by_default = False,
        enumeration = [
            "plain",
            "html"
        ],
        translatable_enumeration = False,
        text_search = False
    )

    sender = schema.CodeBlock(
        language = "python"
    )

    receivers = schema.CodeBlock(
        language = "python",
        required = True
    )

    bcc = schema.CodeBlock(
        language = "python",
        listed_by_default = False
    )

    template_engine = schema.String(
        enumeration = buffet.available_engines.keys(),
        translatable_enumeration = False,
        text_search = False,
        listed_by_default = False
    )

    subject = schema.String(
        translated = True,
        edit_control = "cocktail.html.TextArea"
    )

    body = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea"
    )

    initialization_code = schema.CodeBlock(
        language = "python"
    )

    language = schema.CodeBlock(
        language = "python"
    )

    def send(self, context = None):

        if context is None:
            context = {}

        if context.get("attachments") is None:
            context["attachments"] = {}

        def eval_member(key):
            expr = self.get(key)
            return eval(expr, context.copy()) if expr else None

        # MIME block
        mime_type = self.mime_type
        pos = mime_type.find("/")

        if pos != -1:
            mime_type = mime_type[pos + 1:]

        # Language (python expression)
        language = eval_member("language")
        if language not in self.translations:
            language = None

        with language_context(language):

            # Custom initialization code
            init_code = self.initialization_code
            if init_code:
                exec init_code in context

            # Subject and body (templates)
            if self.template_engine:
                template_engine = buffet.available_engines[self.template_engine]
                engine = template_engine(
                    options = {"mako.output_encoding": self.encoding}
                )

                def render(field_name):
                    markup = self.get(field_name)
                    if markup:
                        template = engine.load_template(
                            "EmailTemplate." + field_name,
                            self.get(field_name)
                        )
                        return engine.render(context, template = template)
                    else:
                        return u""

                subject = render("subject").strip()
                body = render("body")
            else:
                subject = self.subject.encode(self.encoding)
                body = self.body.encode(self.encoding)

            message = MIMEText(body, _subtype = mime_type, _charset = self.encoding)

            # Attachments
            attachments = context.get("attachments")
            if attachments:
                attachments = dict(
                    (cid, attachment)
                    for cid, attachment in attachments.iteritems()
                    if attachment is not None
                )
                if attachments:
                    message_text = message
                    message = MIMEMultipart("related")
                    message.attach(message_text)

                    for cid, attachment in attachments.iteritems():

                        if isinstance(attachment, File):
                            file_path = attachment.file_path
                            file_name = attachment.file_name
                            mime_type = attachment.mime_type
                        else:
                            file_path = attachment
                            file_name = os.path.basename(file_path)
                            mime_type_guess = guess_type(file_path)
                            if mime_type_guess:
                                mime_type = mime_type_guess[0]
                            else:
                                mime_type = "application/octet-stream"

                        main_type, sub_type = mime_type.split('/', 1)
                        message_attachment = MIMEBase(main_type, sub_type)
                        message_attachment.set_payload(open(file_path).read())
                        Encoders.encode_base64(message_attachment)
                        message_attachment.add_header("Content-ID", "<%s>" % cid)
                        message_attachment.add_header(
                            'Content-Disposition',
                            'attachment; filename="%s"' % file_name
                        )
                        message.attach(message_attachment)

            def format_email_address(address, encoding):
                name, address = parseaddr(address)
                name = Header(name, encoding).encode()
                address = address.encode('ascii')
                return formataddr((name, address))

             # Receivers (python expression)
            receivers = eval_member("receivers")
            if receivers:
                receivers = set(r.strip().encode(self.encoding) for r in receivers)

            if not receivers:
                return set()

            message["To"] = ", ".join([
                format_email_address(receiver, self.encoding)
                for receiver in receivers
            ])

            # Sender (python expression)
            sender = eval_member("sender")
            if sender:
                message['From'] = format_email_address(sender, self.encoding)

            # BCC (python expression)
            bcc = eval_member("bcc")
            if bcc:
                receivers.update(r.strip().encode(self.encoding) for r in bcc)

            if subject:
                message["Subject"] = Header(subject, self.encoding).encode()

            message["Date"] = formatdate()

            # Send the message
            smtp = Configuration.instance.connect_to_smtp()
            smtp.sendmail(sender, list(receivers), message.as_string())
            smtp.quit()

        return receivers




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import templates
from woost.models import Extension


translations.define("NoticesExtension",
    ca = u"Avisos",
    es = u"Avisos",
    en = u"Notices"
)

translations.define("NoticesExtension-plural",
    ca = u"Avisos",
    es = u"Avisos",
    en = u"Notices"
)


class NoticesExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet mostrar avisos als visitants de la web.""",
            "ca"
        )
        self.set("description",
            u"""Permite mostrar avisos als visitants de la web.""",
            "es"
        )
        self.set("description",
            u"""Show notices to website visitors.""",
            "en"
        )

    def _load(self):

        from woost.extensions.notices import (
            strings,
            website
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema


class ElementType(schema.String):

    required = True
    default = "div"
    enumeration = [
        "div",
        "section",
        "article",
        "details",
        "aside",
        "figure",
        "header",
        "footer",
        "nav",
        "dd"
    ]

    def translate_value(self, value, language = None, **kwargs):

        translation = schema.String.translate_value(
            self,
            value,
            language,
            **kwargs
        )

        if (
            not translation
            and value
            and self.translatable_enumeration
            and self.enumeration
        ):
            translation = translations("woost.models.ElementType=" + value)

        return translation




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from json import dumps
from woost.models import Configuration

def add_event(
    element,
    category,
    action,
    label = None,
    value = None
):
    api = Configuration.instance.google_analytics_api

    if api == "ga.js":

        event_command = [
            '_trackEvent',
            category,
            action
        ]

        if label is not None:
            event_command.append(label)

        if value is not None:
            event_command.append(value)

        element.add_client_code(
            "jQuery(this).click(function () { _gaq.push(%s); });"
            % dumps(event_command)
        )

    elif api == "universal":

        event_data = {
            "hitType": "event",
            "eventCategory": category,
            "eventAction": action
        }

        if label is not None:
            event_data["eventLabel"] = label

        if value is not None:
            event_data["value"] = value

        element.add_client_code(
            "jQuery(this).click(function () { ga('send', %s); });"
            % dumps(event_data)
        )




#-*- coding: utf-8 -*-
u"""Defines the `Block` model.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from datetime import datetime
from cocktail.pkgutils import import_object
from cocktail.iteration import last
from cocktail.translations import translations, get_language
from cocktail import schema
from cocktail.html import templates, Element
from .item import Item
from .publishable import Publishable
from .style import Style
from .slot import Slot


class Block(Item):

    instantiable = False
    visible_from_root = False
    edit_view = "woost.views.BlockFieldsView"
    type_group = "blocks.content"
    type_groups_order = [
        "blocks.content",
        "blocks.layout",
        "blocks.listings",
        "blocks.social",
        "blocks.forms",
        "blocks.custom"
    ]
    view_class = None
    block_display = "woost.views.BlockDisplay"
    backoffice_heading_view = "woost.views.BackOfficeBlockHeading"

    groups_order = [
        "content",
        "behavior",
        "html",
        "administration"
    ]

    members_order = [
        "heading",
        "heading_type",
        "per_language_publication",
        "enabled",
        "translation_enabled",
        "start_date",
        "end_date",
        "controller",
        "styles",
        "inline_css_styles",
        "html_attributes"
    ]

    heading = schema.String(
        descriptive = True,
        translated = True,
        member_group = "content"
    )

    heading_type = schema.String(
        default = "hidden",
        enumeration = [
            "hidden",
            "hidden_h1",
            "generic",
            "h1",
            "h2",
            "h3",
            "h4",
            "h5",
            "h6",
            "dt",
            "figcaption"
        ],
        required = heading,
        member_group = "content"
    )

    per_language_publication = schema.Boolean(
        required = True,
        default = False,
        member_group = "behavior"
    )

    enabled = schema.Boolean(
        required = True,
        default = True,
        member_group = "behavior"
    )

    translation_enabled = schema.Boolean(
        required = True,
        default = True,
        translated = True,
        member_group = "behavior"
    )

    start_date = schema.DateTime(
        indexed = True,
        affects_cache_expiration = True,
        member_group = "behavior"
    )

    end_date = schema.DateTime(
        indexed = True,
        min = start_date,
        affects_cache_expiration = True,
        member_group = "behavior"
    )

    controller = schema.String(
        member_group = "behavior"
    )

    styles = schema.Collection(
        items = schema.Reference(type = Style),
        relation_constraints = {"applicable_to_blocks": True},
        related_end = schema.Collection(),
        member_group = "html"
    )

    inline_css_styles = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "html"
    )

    html_attributes = schema.String(
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "html"
    )

    initialization = schema.CodeBlock(
        language = "python",
        member_group = "administration"
    )

    def get_block_image(self):
        return self

    def create_view(self):

        if self.view_class is None:
            raise ValueError("No view specified for block %s" % self)

        view = templates.new(self.view_class)
        self.init_view(view)

        if self.controller:
            controller_class = import_object(self.controller)
            controller = controller_class()
            controller.block = self
            controller.view = view
            controller()
            for key, value in controller.output.iteritems():
                setattr(view, key, value)

        initialization = self.initialization
        if initialization:
            code = compile(
                initialization,
                "%s #%d.initialization" % (self.__class__.__name__, self.id),
                "exec"
            )
            exec code in {"block": self, "view": view}

        return view

    def init_view(self, view):
        view.block = self

        block_proxy = self.get_block_proxy(view)
        block_proxy.set_client_param("blockId", self.id)
        block_proxy.add_class("block")

        if self.html_attributes:
            for line in self.html_attributes.split("\n"):
                try:
                    pos = line.find("=")
                    key = line[:pos]
                    value = line[pos + 1:]
                except:
                    pass
                else:
                    block_proxy[key.strip()] = value.strip()

        if self.inline_css_styles:
            for line in self.inline_css_styles.split(";"):
                try:
                    key, value = line.split(":")
                except:
                    pass
                else:
                    block_proxy.set_style(key.strip(), value.strip())

        for style in self.styles:
            block_proxy.add_class(style.class_name)

        block_proxy.add_class("block%d" % self.id)

        if self.qname:
            block_proxy.add_class(self.qname.replace(".", "-"))

        if self.has_heading():
            self.add_heading(view)

        view.depends_on(self)

    def get_block_proxy(self, view):
        return view

    def has_heading(self):
        return bool(self.heading)

    def add_heading(self, view):
        if self.heading_type != "hidden":
            if hasattr(view, "heading"):
                if isinstance(view.heading, Element):
                    if self.heading_type == "hidden_h1":
                        view.heading.tag = "h1"
                        view.heading.set_style("display", "none")
                    elif self.heading_type == "generic":
                        view.heading.tag = "div"
                    else:
                        view.heading.tag = self.heading_type

                    label = self.heading
                    if label:
                        view.heading.append(label)
                else:
                    view.heading = self.heading
            else:
                insert_heading = getattr(view, "insert_heading", None)
                view.heading = self.create_heading()
                if insert_heading:
                    insert_heading(view.heading)
                else:
                    view.insert(0, view.heading)

    def create_heading(self):

        if self.heading_type == "hidden_h1":
            heading = Element("h1")
            heading.set_style("display", "none")
        elif self.heading_type == "generic":
            heading = Element()
        else:
            heading = Element(self.heading_type)

        heading.add_class("heading")

        label = self.heading
        if label:
            heading.append(label)

        return heading

    def is_common_block(self):
        from .configuration import Configuration
        return bool(self.get(Configuration.common_blocks.related_end))

    def is_published(self):

        # Time based publication window
        if self.start_date or self.end_date:
            now = datetime.now()

            # Not published yet
            if self.start_date and now < self.start_date:
                return False

            # Expired
            if self.end_date and now >= self.end_date:
                return False

        if self.per_language_publication:
            return self.translation_enabled
        else:
            return self.enabled

    def _included_in_cascade_delete(self, parent, member):

        if isinstance(parent, Block) and self.is_common_block():
            return False

        return Item._included_in_cascade_delete(self, parent, member)

    def find_publication_slots(self):
        """Iterates over the different slots of publishable elements that
        contain the block.

        @return: An iterable sequence of the slots that contain the block. Each
            slot is represented by a tuple consisting of a L{Publishable} and a
            L{Member<cocktail.schema.member>}.
        """
        visited = set()

        def iter_slots(block):

            for member in block.__class__.members().itervalues():
                if (
                    (block, member) not in visited
                    and isinstance(member, schema.RelationMember)
                    and member.related_type
                ):
                    value = block.get(member)
                    if value is not None:

                        # Yield relations to publishable elements
                        if issubclass(member.related_type, Publishable):
                            if isinstance(member, schema.Collection):
                                for publishable in value:
                                    yield (publishable, member)
                            else:
                                yield (value, member)

                        # Recurse into relations to other blocks
                        elif issubclass(member.related_type, Block):

                            visited.add((block, member))

                            if member.related_end:
                                visited.add((block, member.related_end))

                            if isinstance(member, schema.Collection):
                                for child in value:
                                    for slot in iter_slots(child):
                                        yield slot
                            else:
                                for slot in iter_slots(value):
                                    yield slot

        return iter_slots(self)

    def find_paths(self):
        """Iterates over the different sequences of slots that contain the block.

        @return: A list of lists, where each list represents one of the paths
            that the block descends from. Each entry in a path consists of
            container, slot pair.
        @rtype: list of
            (L{Item<woost.models.item.Item>},
            L{Slot<woost.models.slot.Slot>}) lists
        """
        def visit(block, followed_path):

            paths = []

            for member in block.__class__.members().itervalues():
                related_end = getattr(member, "related_end", None)
                if isinstance(related_end, Slot):
                    parents = block.get(member)
                    if parents:
                        if isinstance(parents, Item):
                            parents = (parents,)
                        for parent in parents:
                            location = (parent, related_end)
                            if location not in followed_path:
                                paths.extend(
                                    visit(parent, [location] + followed_path)
                                )

            # End of the line
            if not paths and followed_path:
                paths.append(followed_path)

            return paths

        return visit(self, [])

    @property
    def name_prefix(self):
        return "block%d." % self.id

    @property
    def name_suffix(self):
        return None

    def replace_with(self, replacement):
        """Removes this block from all slots, putting another block in the same
        position.

        @param replacement: The block to insert.
        @type replacement: L{Block}
        """
        for member in self.__class__.members().itervalues():
            related_end = getattr(member, "related_end", None)
            if isinstance(related_end, Slot):
                for container in self.get(member):
                    slot_content = container.get(related_end)
                    slot_content[slot_content.index(self)] = replacement




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.html import templates
from .rendering import ImageFactory
from .block import Block
from .slot import Slot
from .elementtype import ElementType
from .blockutils import create_block_views


class SlideShowBlock(Block):

    instantiable = True
    view_class = "woost.views.SlideShowBlockView"

    groups_order = [
        "content",
        "transition_settings",
        "controls",
        "behavior",
        "html",
        "administration"
    ]

    members_order = [
        "element_type",
        "slides",
        "autoplay",
        "interval",
        "transition_effect",
        "transition_duration",
        "navigation_controls",
        "bullet_controls",
        "bullet_view_class",
        "bullet_image_factory"
    ]

    element_type = ElementType(
        enumeration = list(ElementType.enumeration),
        member_group = "content"
    )

    for et in ("nav", "dd"):
        try:
            element_type.enumeration.remove(et)
        except:
            pass

    slides = Slot()

    autoplay = schema.Boolean(
        required = True,
        default = True,
        member_group = "transition_settings"
    )

    interval = schema.Integer(
        required = autoplay,
        default = 3000,
        min = 0,
        member_group = "transition_settings"
    )

    transition_effect = schema.String(
        required = True,
        default = "fade",
        enumeration = ["fade", "topBottomSlide"],
        member_group = "transition_settings"
    )

    transition_duration = schema.Integer(
        required = True,
        default = 500,
        min = 0,
        member_group = "transition_settings"
    )

    navigation_controls = schema.Boolean(
        required = True,
        default = False,
        member_group = "controls"
    )

    bullet_controls = schema.Boolean(
        required = True,
        default = False,
        member_group = "controls"
    )

    bullet_view_class = schema.String(
        required = True,
        default = "woost.views.SlideShowButtonBullet",
        enumeration = [
            "woost.views.SlideShowButtonBullet",
            "woost.views.SlideShowTextBullet",
            "woost.views.SlideShowImageBullet",
            "woost.views.SlideShowTextAndImageBullet"
        ],
        member_group = "controls"
    )

    bullet_image_factory = schema.Reference(
        type = ImageFactory,
        related_end = schema.Collection(),
        required = True,
        default = schema.DynamicDefault(
            lambda: ImageFactory.get_instance(identifier = "image_gallery_thumbnail")
        ),
        member_group = "controls"
    )

    def init_view(self, view):

        Block.init_view(self, view)

        view.tag = self.element_type
        view.autoplay = self.autoplay
        view.interval = self.interval
        view.transition_effect = self.transition_effect
        view.transition_duration = self.transition_duration
        view.navigation_controls = self.navigation_controls
        view.bullet_controls = self.bullet_controls
        view.bullet_view_class = self.bullet_view_class
        view.bullet_image_factory = self.bullet_image_factory

        for block_view in create_block_views(self.slides):
            view.slides.append(block_view)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations


# YouTubeVideo
#------------------------------------------------------------------------------
translations.define("YouTubeVideo",
    ca = u"Vídeo de YouTube",
    es = u"Video de YouTube",
    en = u"YouTube video"
)

translations.define("YouTubeVideo-plural",
    ca = u"Vídeos de YouTube",
    es = u"Videos de YouTube",
    en = u"YouTube videos"
)

translations.define("YouTubeVideo.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("YouTubeVideo.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("YouTubeVideo.video_id",
    ca = u"Identificador del vídeo",
    es = u"Identificador del video",
    en = u"Video ID"
)

# YouTubeVideoRenderer
#------------------------------------------------------------------------------
translations.define("YouTubeVideoRenderer",
    ca = u"Pintador de fotogrames pels vídeos de YouTube",
    es = u"Pintador de fotogramas para los videos de YouTube",
    en = u"YouTube videos still renderer"
)

translations.define("YouTubeVideoRenderer-plural",
    ca = u"Pintadors de fotogrames pels vídeos de YouTube",
    es = u"Pintadores de fotogramas para los videos de YouTube",
    en = u"YouTube videos still renderers"
)

# VideoPlayerSettings
#------------------------------------------------------------------------------
translations.define("VideoPlayerSettings.youtube",
    ca = u"YouTube",
    es = u"YouTube",
    en = u"YouTube"
)

translations.define("VideoPlayerSettings.youtube_allow_fullscreen",
    ca = u"Permetre pantalla completa",
    es = u"Permitir pantalla completa",
    en = u"Allow fullscreen"
)

translations.define("VideoPlayerSettings.youtube_show_info",
    ca = u"Sobreimpressionar el títol del vídeo",
    es = u"Sobreimpresionar el título del video",
    en = u"Overlay the video title"
)

translations.define("VideoPlayerSettings.youtube_show_related_videos",
    ca = u"Mostrar vídeos relacionats",
    es = u"Mostrar videos relacionados",
    en = u"Show related videos"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .block import Block
from .slot import Slot


def type_is_block_container(cls):

    for member in cls.members().itervalues():
        if (
            isinstance(member, schema.RelationMember)
            and member.related_type
            and issubclass(member.related_type, Block)
            and member.visible and member.editable
        ):
            return True

    return any(
        type_is_block_container(subclass)
        for subclass in cls.derived_schemas(recursive = False)
    )

def find_blocks(obj, slots = None):

    if isinstance(obj, Block):
        yield obj

    if slots is None:
        slots = (member
                for member in obj.__class__.members().itervalues()
                if isinstance(member, Slot))

    for slot in slots:
        blocks = obj.get(slot)
        if blocks is not None:
            for block in blocks:
                for descendant in find_blocks(block):
                    yield descendant

def add_block(block, parent, slot, positioning = "append", anchor = None):
    if isinstance(slot, schema.Reference):
        parent.set(slot, block)
    elif isinstance(slot, schema.Collection):
        collection = parent.get(slot)
        if positioning == "append":
            schema.add(collection, block)
        elif positioning == "before":
            collection.insert(collection.index(anchor), block)
        elif positioning == "after":
            collection.insert(collection.index(anchor) + 1, block)
        else:
            raise ValueError("Invalid block positioning: %s" % positioning)

def create_block_views(blocks):
    for block in blocks:
        if block.is_published():
            view = block.create_view()
            if view is not None:
                yield view




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail.translations import translations


# TV3ALaCartaVideo
#------------------------------------------------------------------------------
translations.define("TV3ALaCartaVideo",
    ca = u"Vídeo de TV3 a la carta",
    es = u"Video de TV3 a la carta",
    en = u"TV3 a la carta video"
)

translations.define("TV3ALaCartaVideo-plural",
    ca = u"Vídeos de TV3 a la carta",
    es = u"Videos de TV3 a la carta",
    en = u"TV3 a la carta videos"
)

translations.define("TV3ALaCartaVideo.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("TV3ALaCartaVideo.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("TV3ALaCartaVideo.video_id",
    ca = u"Identificador del vídeo",
    es = u"Identificador del video",
    en = u"Video ID"
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
import cherrypy
from cocktail.modeling import cached_getter
from woost.controllers.publishablecontroller import PublishableController


class DocumentController(PublishableController):
    """A controller that serves rendered pages."""

    def __call__(self, **kwargs):

        # Document specified redirection
        document = self.context["publishable"]

        if document.redirection_mode:

            redirection_target = document.find_redirection_target()

            if redirection_target is None:
                raise cherrypy.NotFound()

            raise cherrypy.HTTPRedirect(redirection_target.get_uri())

        # No redirection, serve the document normally
        return PublishableController.__call__(self)

    @cached_getter
    def page_template(self):
        template = self.context["publishable"].template

        if template is None:
            raise cherrypy.NotFound()

        return template

    @cached_getter
    def view_class(self):
        return self.page_template.identifier




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from ZODB.broken import Broken
from cocktail import schema
from cocktail.persistence import datastore
from cocktail.persistence.utils import (
    is_broken,
    remove_broken_type as cocktail_remove_broken_type
)
from .item import Item
from .configuration import Configuration
from .permission import ContentPermission
from .changesets import Change
from .role import Role


def remove_broken_type(
    full_name,
    existing_bases = (),
    relations = (),
    excluded_relations = None,
    languages = None
):
    if languages is None:
        languages = Configuration.instance.languages

    if excluded_relations is None:
        excluded_relations = (Change.target,)

    cocktail_remove_broken_type(
        full_name,
        existing_bases = existing_bases,
        relations = relations,
        excluded_relations = excluded_relations,
        languages = languages
    )

    for role in Role.select():
        for cls in list(role.hidden_content_types):
            if (
                issubclass(cls, Broken)
                and cls.__module__ + "." + cls.__name__ == full_name
            ):
                role.hidden_content_types.remove(cls)

    for permission in ContentPermission.select():
        matching_items = getattr(permission, "matching_items", None)
        if matching_items:
            type = matching_items.get("type")
            if type and type == full_name:
                permission.delete()

def delete_history():

    for item in Item.select():
        if not is_broken(item):
            try:
                del item._changes
            except AttributeError:
                pass

    for key in list(datastore.root):
        if key.startswith("woost.models.changesets."):
            del datastore.root[key]

def restore_deleted_item(obj):
    if obj.insert():
        for member in obj.__class__.iter_members():
            if (
                isinstance(member, schema.Collection)
                and obj.get(member) is None
            ):
                obj.set(member, member.produce_default(instance = obj))




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.controllers.location import Location
from woost.models import Extension

translations.define("PaymentsExtension",
    ca = u"Pagaments",
    es = u"Pagos",
    en = u"Payments"
)

translations.define("PaymentsExtension-plural",
    ca = u"Pagaments",
    es = u"Pagos",
    en = u"Payments"
)


class PaymentsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Afageix suport per pagaments en línia.""",
            "ca"
        )
        self.set("description",
            u"""Añade soporte para pagos en linea.""",
            "es"
        )
        self.set("description",
            u"""Adds support for online payments.""",
            "en"
        )

    def _load(self):

        # Import the extension's models
        from woost.extensions.payments.paymentgateway import PaymentGateway
        from woost.extensions.payments import (
            strings,
            dummypaymentgateway,
            pasat4b,
            sis,
            paypal,
            transactionnotifiedtrigger
        )

        # Setup payment controllers
        from woost.controllers.cmscontroller import CMSController
        from woost.extensions.payments.paymentrootcontroller \
            import PaymentRootController

        CMSController.payments = PaymentRootController




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.controllers import (
    Controller,
    request_property,
    Form,
    FormProcessor
)
from woost.models import get_current_website
from woost.extensions.ecommerce.basket import Basket
from woost.extensions.ecommerce.orderstepcontroller import ProceedForm


class CheckoutController(FormProcessor, Controller):

    class CheckoutForm(ProceedForm):

        @request_property
        def model(self):
            return Basket.get().get_public_schema()

        @request_property
        def source_instance(self):
            return Basket.get()

        @request_property
        def schema(self):
            schema = ProceedForm.schema(self)
            payment_type = schema.get_member("payment_type")
            if payment_type:
                payment_type.enumeration = \
                    get_current_website().ecommerce_payment_types
            return schema

        def submit(self):
            ProceedForm.submit(self)
            Basket.store()




#-*- coding: utf-8 -*-
u"""
Site specific views.
"""
from pkg_resources import resource_filename
from cocktail.html import renderers, resource_repositories
renderers.default_renderer = renderers.html5_renderer

resource_repositories.define(
    "_PROJECT_MODULE_",
    "/_PROJECT_MODULE__resources",
    resource_filename("_PROJECT_MODULE_.views", "resources")
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from shutil import rmtree
from tempfile import mkdtemp
from subprocess import Popen, PIPE
from time import time, sleep
from PIL import Image
from cocktail import schema
from woost.models.file import File
from woost.models.rendering.renderer import Renderer


class PDFRenderer(Renderer):
    """A content renderer that handles pdf files."""

    instantiable = True

    command = schema.String(
        required = True
    )

    timeout = schema.Integer(
        required = True,
        default = 20
    )

    timeout_size_factor = schema.Float(
        default = 10.0
    )

    def can_render(self, item, **parameters):
        return (
            self.command
            and isinstance(item, File)
            and item.resource_type == "document"
            and item.file_name.split(".")[-1].lower() == "pdf"
        )

    def render(self, item, page = 1, **parameters):

        timeout = self.timeout

        # Increase the timeout for bigger files
        if self.timeout_size_factor:
            size = item.file_size
            if size:
                timeout += size / (self.timeout_size_factor * 1024 * 1024)

        RESOLUTION = 0.25
        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command = self.command % {
                "source": item.file_path,
                "dest": temp_image_file,
                "page": page
            }

            p = Popen(command, shell=True, stdout=PIPE)
            start = time()

            while p.poll() is None:
                if time() - start > timeout:
                    p.terminate()
                    raise IOError(
                        "PDF rendering timeout: '%s' took more than "
                        "%.2f seconds"
                        % (command, timeout)
                    )
                sleep(RESOLUTION)

            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)


class PDFTimeoutError(IOError):
    """An exception raised when a PDF rendering operation takes too long to
    complete.
    """


try:
    p = Popen(["which", "gs"], stdout=PIPE)
    output = p.communicate()[0].replace("\n", "")
except:
    pass
else:
    if output:
        PDFRenderer.command.default = (
            output + ' -dFirstPage=%(page)s -dLastPage=%(page)s'
            ' -sDEVICE=jpeg -o %(dest)s -dJPEGQ=95 -r150x150 %(source)s'
        )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
import cherrypy
from decimal import Decimal
from simplejson import dumps
from cocktail.modeling import underscore_to_capital
from cocktail.translations import (
    translations,
    set_language,
    language_context
)
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import (
    Configuration,
    Extension,
    Publishable,
    Page,
    Template,
    Controller,
    EmailTemplate,
    User,
    TextBlock,
    CustomBlock
)
from woost.models.rendering import ImageFactory, Thumbnail
from woost.models.triggerresponse import SendEmailTriggerResponse
from woost.controllers import CMSController

translations.define("ECommerceExtension",
    ca = u"Comerç online",
    es = u"Comercio online",
    en = u"E-commerce"
)

translations.define("ECommerceExtension-plural",
    ca = u"Comerç online",
    es = u"Comercio online",
    en = u"E-commerce"
)


class ECommerceExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet vendre productes des del lloc web.""",
            "ca"
        )
        self.set("description",
            u"""Permite vender productos des del sitio web.""",
            "es"
        )
        self.set("description",
            u"""Allows selling products online.""",
            "en"
        )

    def _load(self):

        from woost.extensions.ecommerce import (
            strings,
            typegroups,
            website,
            ecommerceproduct,
            ecommerceorder,
            ecommercepurchase,
            ecommercebillingconcept,
            ecommerceordercompletedtrigger,
            orderstepscontainerblock,
            ecommerceproductlisting,
        )

        # If the payments extension is enabled, setup the payment gateway for
        # the shop
        from woost.extensions.payments import PaymentsExtension
        payments_ext = PaymentsExtension.instance

        if payments_ext.enabled:
            self._setup_payment_gateway()

        self._setup_ecommerce_state()

        # Create the pages for the shop the first time the extension runs
        self.install()

    def _setup_ecommerce_state(self):

        from woost.extensions.ecommerce.basket import Basket

        @cherrypy.expose
        def ecommerce_state(self):

            cherrypy.response.headers["Content-Type"] = "text/javascript"
            order = Basket.get(create_new = False)
            if order:
                order.update_cost()

            def format_amount(amount):
                return str(amount.quantize(Decimal("1.00")))

            return "cocktail.declare('woost.ecommerce'); woost.ecommerce = %s;" % dumps(
                {
                    "order": {
                        "id": order.id,
                        "total": format_amount(order.total),
                        "count_label": translations(
                            "woost.extensions.ecommerce.BasketIndicator",
                            count = order.count_units()
                        ),
                        "purchases": [{
                            "id": purchase.id,
                            "product": {
                                "id": purchase.product.id,
                                "label": translations(purchase.product)
                            },
                            "quantity": purchase.quantity,
                            "total": format_amount(purchase.total)
                        }
                        for purchase in order.purchases]
                    }
                }
                if order else {
                    "order": {
                        "count_label": translations(
                            "woost.extensions.ecommerce.BasketIndicator",
                            count = 0
                        )
                    }
                }
            )

        CMSController.ecommerce_state = ecommerce_state

    def _setup_payment_gateway(self):

        from tpv import (
            Currency,
            Payment,
            PaymentItem,
            PaymentNotFoundError
        )
        from woost.extensions.payments.paymentgateway import PaymentGateway
        from woost.extensions.payments.transactionnotifiedtrigger \
            import launch_transaction_notification_triggers
        from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder
        from woost.extensions.payments import PaymentsExtension
        from woost.extensions.payments.paypal import PayPalPaymentGateway

        ecommerce_ext = self

        def get_payment(self, payment_id):

            order = ECommerceOrder.get_instance(int(payment_id))

            if order is None:
                raise PaymentNotFoundError(payment_id)

            payment = Payment()
            payment.id = order.id
            payment.description = order.get_description_for_gateway()
            payment.amount = order.total
            payment.order = order
            payment.currency = Currency(self.currency)

            # PayPal form data
            if isinstance(self, PayPalPaymentGateway):
                paypal_form_data = {}
                if order.address:
                    paypal_form_data["address1"] = order.address

                if order.town:
                    paypal_form_data["city"] = order.town

                if order.country and order.country.code:
                    paypal_form_data["country"] = order.country.code

                if order.postal_code:
                    paypal_form_data["zip"] = order.postal_code

                if order.language:
                    paypal_form_data["lc"] = order.language

                payment.paypal_form_data = paypal_form_data

            for purchase in order.purchases:
                payment.add(PaymentItem(
                    reference = str(purchase.product.id),
                    description = translations(purchase.product),
                    units = purchase.quantity,
                    price = purchase.total
                ))

            return payment

        PaymentGateway.get_payment = get_payment

        def receive_order_payment(event):
            payment = event.payment
            order = payment.order
            set_language(order.language)
            order.status = payment.status
            order.gateway_parameters = payment.gateway_parameters

        def commit_order_payment(event):
            datastore.commit()

        events = PaymentGateway.transaction_notified
        pos = events.index(launch_transaction_notification_triggers)
        events.insert(pos, receive_order_payment)
        events.insert(pos + 2, commit_order_payment)

    def _install(self):

        from woost.extensions.ecommerce.ecommerceproductlisting import ECommerceProductListing

        website = Configuration.instance.websites[0]

        catalog = self._create_document("catalog")
        catalog.blocks.append(ECommerceProductListing())
        catalog.insert()
        website.ecommerce_default_catalog = catalog

        for child_name in (
            "basket",
            "checkout",
            "summary"
        ):
            child = self._create_document(child_name)
            self.__add_ecommerce_step_blocks(child, child_name)
            child.hidden = True
            child.parent = catalog
            child.insert()
            website.ecommerce_order_steps.append(child)

        for child_name in (
            "success",
            "failure"
        ):
            child = self._create_document(child_name)
            child.hidden = True
            child.parent = catalog
            child.insert()

        self._create_controller("product").insert()
        self._create_ecommerceorder_completed_trigger().insert()
        self._create_incoming_order_trigger().insert()
        self._create_image_factories()

    def _create_document(self, name,
        cls = Page,
        template = None,
        controller = None):

        document = cls()
        document.qname = "woost.extensions.ecommerce.%s_page" % name

        self.__translate_field(document, "title")

        if isinstance(document, Page):
            self.__add_text_block(document, "body")

        return document

    def _create_controller(self, name):
        controller = Controller( )
        controller.qname = "woost.extensions.ecommerce.%s_controller" % name
        self.__translate_field(controller, "title")
        controller.python_name = \
            "woost.extensions.ecommerce.%scontroller.%sController" % (
                name,
                underscore_to_capital(name)
            )
        return controller

    def _create_ecommerceorder_completed_trigger(self):
        from woost.extensions.ecommerce.ecommerceordercompletedtrigger import \
            ECommerceOrderCompletedTrigger
        trigger = ECommerceOrderCompletedTrigger( )
        trigger.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_trigger"
        self.__translate_field(trigger, "title")
        Configuration.instance.triggers.append(trigger)
        trigger.condition = "target.customer and not target.customer.anonymous and target.customer.email"
        trigger.matching_items = {'type': u'woost.extensions.ecommerce.ecommerceorder.ECommerceOrder'}

        # EmailTemplate
        template = EmailTemplate()
        template.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_emailtemplate"
        self.__translate_field(template, "title")
        template.sender = '"%s"' % User.require_instance(
            qname = "woost.administrator"
        ).email
        template.receivers = '[items[0].customer.email]'
        template.initialization_code = """
from woost.models import Configuration
logo = Configuration.instance.get_setting("logo")
if logo:
    attachments["logo"] = logo
"""
        template.template_engine = "mako"

        for language in Configuration.instance.languages:
            with language_context(language):
                template.subject = template.title
                template.body = """
<%
from cocktail.controllers import Location
from cocktail.html import templates

order = items[0]
order_summary = templates.new("woost.extensions.ecommerce.OrderConfirmationMessage")
order_summary.order = order
%>

<html>
    <head>
        <base href="@{unicode(Location.get_current_host())}"/>
    </head>
    <body>
        ${order_summary.render()}
    </body>
</html>
"""

        # Response
        response = SendEmailTriggerResponse()
        response.qname = \
            "woost.extensions.ecommerce.ecommerceorder_completed_response"
        response.email_template = template

        trigger.responses = [response]
        return trigger

    def _create_incoming_order_trigger(self):
        from woost.extensions.ecommerce.incomingordertrigger import \
            IncomingOrderTrigger
        trigger = IncomingOrderTrigger( )
        trigger.qname = "woost.extensions.ecommerce.incoming_order.trigger"
        self.__translate_field(trigger, "title")
        Configuration.instance.triggers.append(trigger)
        trigger.matching_items = {'type': u'woost.extensions.ecommerce.ecommerceorder.ECommerceOrder'}

        # EmailTemplate
        template = EmailTemplate()
        template.qname = \
            "woost.extensions.ecommerce.incoming_order.email_template"
        self.__translate_field(template, "title")
        admin = User.require_instance(qname = "woost.administrator")
        template.sender = repr(admin.email)
        template.receivers = repr([admin.email])
        template.template_engine = "mako"

        for language in Configuration.instance.languages:
            with language_context(language):
                template.subject = template.title
                template.body = """
<%
from cocktail.translations import translations
from woost.models import Publishable

order = items[0]
bo = Publishable.require_instance(qname = "woost.backoffice")
edit_url = bo.get_uri(host = ".", path = ["content", str(order.id)])
%>

<html>
    <body>
        <a href="${edit_url}">${translations('woost.extensions.ecommerce.incoming_order.edit_link')}
        </a>
    </body>
</html>
"""

        # Response
        response = SendEmailTriggerResponse()
        response.email_template = template
        trigger.responses = [response]

        return trigger

    def _create_image_factories(self):
        thumbnail = ImageFactory()
        thumbnail.qname = \
            "woost.extensions.ecommerce.ecommerce_basket_thumbnail"
        self.__translate_field(thumbnail, "title")
        thumbnail.identifier = "ecommerce_basket_thumbnail"
        thumbnail.effects = [Thumbnail(width = "75", height = "75")]
        thumbnail.insert()

    def __translate_field(self, obj, key):
        for language in Configuration.instance.languages:
            with language_context(language):
                value = translations("%s.%s" % (obj.qname, key))
                if value:
                    obj.set(key, value)

    def __add_text_block(
        self,
        document,
        key,
        style = None,
        element_type = None,
        slot = "blocks"
    ):
        block = TextBlock()
        if slot:
            blocks = document.get(slot)
        else:
            blocks = document.blocks

        has_text = False
        for language in Configuration.instance.languages:
            with language_context(language):
                value = translations("%s.%s" % (document.qname, key))
                if value:
                    block.set("text", value)
                    has_text = True

        if not has_text:
            return

        if element_type:
            block.element_type = element_type

        if style:
            block.styles.append(style)

        blocks.append(block)
        block.insert()

    def __add_ecommerce_step_blocks(
        self,
        document,
        key
    ):
        from woost.extensions.ecommerce.orderstepscontainerblock import \
        OrderStepsContainerBlock

        order_steps = OrderStepsContainerBlock()
        block = CustomBlock()
        block.qname = "woost.extensions.ecommerce.%s_block" % key
        self.__translate_field(block, "heading")
        block.view_class = \
            "woost.extensions.ecommerce.%sBlockView" % key.capitalize()
        block.controller = \
            "woost.extensions.ecommerce.%scontroller.%sController" % (
                key, key.capitalize()
            )
        order_steps.blocks.append(block)
        document.blocks.append(order_steps)
        order_steps.insert()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2009
"""
from cocktail.translations import get_language
from cocktail import schema
from cocktail.controllers import get_parameter, request_property
from woost.models import Publishable, Block
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class PreviewController(EditController):

    section = "preview"

    @request_property
    def previewed_item(self):
        return self.stack_node.item

    @request_property
    def preview_publishable(self):

        publishable = get_parameter(
            schema.Reference("preview_publishable",
                type = Publishable
            )
        )

        if publishable is not None:
            return publishable

        for node in reversed(self.edit_stack):
            item = getattr(node, "item", None)
            if item is not None and isinstance(item, Publishable):
                return item

        if isinstance(self.previewed_item, Block):
            for path in self.previewed_item.find_paths():
                container = path[0][0]
                if isinstance(container, Publishable):
                    return container

    @request_property
    def preview_language(self):
        return get_parameter(
            schema.String("preview_language",
                default = get_language()
            )
        )

    @request_property
    def view_class(self):
        return self.stack_node.item.preview_view

    @request_property
    def output(self):
        output = EditController.output(self)
        output.update(
            selected_action = get_user_action("preview"),
            previewed_item = self.previewed_item,
            preview_publishable = self.preview_publishable,
            preview_language = self.preview_language,
            item_translations = self.stack_node.item_translations
        )
        return output




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from cocktail.html import templates
from cocktail.typemapping import TypeMapping
from woost.models.item import Item
from woost.models.file import File


class VideoPlayerSettings(Item):

    instantiable = True
    visible_from_root = False
    player_initialization = TypeMapping()

    members_order = [
        "title", "width", "height", "autoplay", "show_player_controls"
    ]

    title = schema.String(
        listed_by_default = False,
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    width = schema.Integer(
        required = True,
        default = 480,
        listed_by_default = False
    )

    height = schema.Integer(
        required = True,
        default = 385,
        listed_by_default = False
    )

    autoplay = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False
    )

    show_player_controls = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False
    )

    def create_player(self, video):
        player = templates.new(video.video_player)
        player_initialization = self.player_initialization.get(video.__class__)
        if player_initialization:
            player_initialization(player, self, video)
        return player


def player_initialization(player, settings, video):
    player.width = settings.width
    player.height = settings.height
    player.autoplay = settings.autoplay
    player.show_player_controls = settings.show_player_controls
    player.sources = ((video.get_uri(), video.mime_type),)

VideoPlayerSettings.player_initialization[File] = player_initialization

